package edu.stanford.nlp.parser.lexparser;

import java.util.regex.Pattern;

/**
 * Unknown word signatures for Arabic Treebank (unvocalized Buckwalter).
 * @author Roger Levy (rog@csli.stanford.edu)
 *
 */
class ArabicUnknownWordSignatures {

  static boolean allDigitPlus(String word) {
    boolean allDigitPlus = true;
    boolean seenDigit = false;
    for (int i = 0, wlen = word.length(); i < wlen; i++) {
      char ch = word.charAt(i);
      if (Character.isDigit(ch)) {
        seenDigit = true;
      } else if (ch == '-' || ch == '.' || ch == ',') {
      } else {
        allDigitPlus = false;
      }
    }
    return allDigitPlus && seenDigit;
  }
  
  private static Pattern adjectivalSuffixPattern = Pattern.compile("yy[n]?$");

  static String likelyAdjectivalSuffix(String word) {
    if(adjectivalSuffixPattern.matcher(word).find())
      return "-AdjSuffix";
    else
      return "";
  }
  
  private static Pattern singularPastTenseSuffixPattern = Pattern.compile("t$"); 
  private static Pattern pluralFirstPersonPastTenseSuffixPattern = Pattern.compile("nA$"); 
  private static Pattern pluralThirdPersonMasculinePastTenseSuffixPattern = Pattern.compile("w$");
  // private static Pattern pluralThirdPersonFemininePastTenseSuffixPattern = Pattern.compile("n$"); // could be used but doesn't seem very discriminating 
  // there doesn't seem to be second-person marking in the corpus, just first and non-first (?)
  static String pastTenseVerbNumberSuffix(String word) {
    if(singularPastTenseSuffixPattern.matcher(word).find())
      return "-PV.sg";
    if(pluralFirstPersonPastTenseSuffixPattern.matcher(word).find())
      return "-PV.pl1";
    if(pluralThirdPersonMasculinePastTenseSuffixPattern.matcher(word).find())
      return "-PV.pl3m";
    else 
      return ""; 
  }
  
  private static Pattern pluralThirdPersonMasculinePresentTenseSuffixPattern = Pattern.compile("wn$");
  static String presentTenseVerbNumberSuffix(String word) {
    return pluralThirdPersonMasculinePresentTenseSuffixPattern.matcher(word).find() ? "-IV.pl3m" : "";
  }
  
  private static Pattern taaMarbuuTaSuffixPattern = Pattern.compile("p$"); // almost always ADJ or NOUN 
  static String taaMarbuuTaSuffix(String word) {
    return taaMarbuuTaSuffixPattern.matcher(word).find() ? "-taaMarbuuTa" : "";
  }
  
  private static Pattern abstractionNounSuffixPattern = Pattern.compile("yp$"); // ironically, this seems to be a better indicator of ADJ than of NOUN. 
  static String abstractionNounSuffix(String word) {
    return taaMarbuuTaSuffixPattern.matcher(word).find() ? "-AbstractionSuffix" : "";
  }
  
  
}
