package edu.stanford.nlp.trees.tregex.tsurgeon;

import edu.stanford.nlp.trees.Tree;
import edu.stanford.nlp.trees.tregex.TregexMatcher;

import java.util.Map;
import java.util.HashMap;

/**
 * An abstract class for patterns to manipulate {@link Tree}s when successfully matched on with a {@link TregexMatcher}.
 *
 * @author Roger Levy
 */
public abstract class TsurgeonPattern {

  protected static final TsurgeonPattern[] EMPTY_TSURGEON_ARRAY = new TsurgeonPattern[] {};

  TsurgeonPatternRoot root;
  String label;
  TsurgeonPattern[] children;

  TsurgeonPattern(String label, TsurgeonPattern[] children) {
    this.label = label;
    this.children = children;
  }

  protected void setRoot(TsurgeonPatternRoot root) {
    this.root = root;
    for(TsurgeonPattern child : children)
      child.setRoot(root);
  }

  public String toString() {
    String result =  label;
    if(children.length > 0) {
      result += "(";
      for(int i = 0; i < children.length; i++) {
        result += children[i].toString();
        if(i < children.length-1) 
          result += ", ";
      }
      result += ")";
    }
    return result;
  }

  /**
   * Evaluates the pattern against a {@link Tree} and a {@link TregexMatcher} that has been successfully matched against the tree.
   * @param t the {@link Tree} that has been matched upon; typically this tree will be destructively modified.
   * @param m the successfully matched {@link TregexMatcher}
   * @return some node in the tree; depends on implementation and use of the specific subclass.
   */
  public abstract Tree evaluate(Tree t, TregexMatcher m);

}
